@@ -1,9 +1,12 @@
 /*
- This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2008.
+ This is part of the OTF library. Copyright by ZIH, TU Dresden 2005-2012.
  Authors: Andreas Knuepfer, Holger Brunst, Ronny Brendel, Thomas Kriebitzsch
+ also: patches by Rainer Keller, thanks a lot!
 */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 
 #include "OTF_Platform.h"
@@ -11,6 +14,9 @@
 #include "OTF_Platform.h"
 #include "OTF_Reader.h"
 #include "OTF_Parse.h"
+#include "OTF_Errno.h"
+
+#include "OTF_Keywords.h"
 
 
 /** constructor - internal use only */
@@ -33,11 +39,12 @@ int OTF_RStream_init( OTF_RStream* rstream ) {
 	rstream->eventBuffer= NULL;
 	rstream->snapsBuffer= NULL;
 	rstream->statsBuffer= NULL;
+	rstream->markerBuffer= NULL;
 
 	rstream->buffersizes= 1024*1024;
 	
 #ifdef HAVE_ZLIB
-	rstream->zbuffersizes= 1024*10;
+	rstream->zbuffersizes= OTF_ZBUFFER_DEFAULTSIZE;
 #endif /* HAVE_ZLIB */
 
 	rstream->manager= NULL;
@@ -65,11 +72,9 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close defbuffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */		
 		}
 		rstream->defBuffer= NULL;
 	}
@@ -80,11 +85,10 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close event buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */		
+	
 		}
 		rstream->eventBuffer= NULL;
 	}
@@ -95,11 +99,10 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close snapshots buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */		
+
 		}
 		rstream->snapsBuffer= NULL;
 	}
@@ -110,15 +113,28 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 		ret &= tmpret;
 		if( 0 == tmpret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"cannot close statistics buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */		
+
 		}
 		rstream->statsBuffer= NULL;
 	}
 
+	if ( NULL != rstream->markerBuffer ) {
+
+		tmpret= OTF_RBuffer_close( rstream->markerBuffer );
+		ret &= tmpret;
+		if( 0 == tmpret ) {
+			
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"cannot close defbuffer.\n",
+					__FUNCTION__, __FILE__, __LINE__ );
+
+		}
+		rstream->markerBuffer= NULL;
+	}
+
 	return ret;
 }
 
@@ -126,14 +142,24 @@ int OTF_RStream_finish( OTF_RStream* rstream ) {
 OTF_RStream* OTF_RStream_open( const char* namestub, uint32_t id, OTF_FileManager* manager ) {
 
 
-	OTF_RStream* ret= (OTF_RStream*) malloc( sizeof(OTF_RStream) );
+	OTF_RStream* ret;
+
+	/* Check the input parameters */
+	if( NULL == manager ) {
+		
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+				"manager has not been specified.\n",
+				__FUNCTION__, __FILE__, __LINE__ );
+
+		return NULL;
+	}
+
+	ret= (OTF_RStream*) malloc( sizeof(OTF_RStream) );
 	if( NULL == ret ) {
 		
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"no memory left.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
-#		endif /* OTF_VERBOSE */
 
 		return NULL;
 	}
@@ -143,19 +169,6 @@ OTF_RStream* OTF_RStream_open( const char* namestub, uint32_t id, OTF_FileManage
 	ret->namestub= strdup( namestub );
 	ret->id= id;
 
-	if( NULL == manager ) {
-		
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-				"manager has not been specified.\n",
-				__FUNCTION__, __FILE__, __LINE__ );
-#		endif /* OTF_VERBOSE */
-
-		free( ret );
-		ret= NULL;
-
-		return NULL;
-	}
 	ret->manager= manager;
 
 	/* leave buffers alone, they are allocated on demand */
@@ -171,11 +184,9 @@ int OTF_RStream_close( OTF_RStream* rstream ) {
 	
 	if( NULL == rstream ) {
 		
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"rstream has not been specified.\n",
 				__FUNCTION__, __FILE__, __LINE__ );
-#		endif /* OTF_VERBOSE */
 
 		return 0;
 	}
@@ -183,6 +194,7 @@ int OTF_RStream_close( OTF_RStream* rstream ) {
 	ret &= OTF_RStream_finish( rstream );
 
 	free( rstream );
+	rstream = NULL;
 
 	return ret;
 }
@@ -200,17 +212,16 @@ OTF_RBuffer* OTF_RStream_getDefBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_DEF, 0, NULL );
 		if( NULL == filename ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 
 			return NULL;
 		}
 
 		rstream->defBuffer= OTF_RBuffer_open( filename, rstream->manager );
 		free( filename );
+		filename = NULL;
 
 		if ( NULL == rstream->defBuffer ) {
 
@@ -227,6 +238,24 @@ OTF_RBuffer* OTF_RStream_getDefBuffer( OTF_RStream* rstream ) {
 }
 
 
+OTF_RBuffer* OTF_RStream_setDefBuffer( OTF_RStream* rstream, OTF_RBuffer* rbuffer ) {
+
+
+    OTF_RBuffer* old= rstream->defBuffer;
+    rstream->defBuffer= rbuffer;
+
+    /* the following is important because otherwise the buffer sizes stay 0 
+    which leads to weird problems down the road */
+
+    OTF_RBuffer_setSize( rstream->defBuffer, rstream->buffersizes );
+#ifdef HAVE_ZLIB
+    OTF_RBuffer_setZBufferSize( rstream->defBuffer, rstream->zbuffersizes );
+#endif /* HAVE_ZLIB */
+
+    return old;
+}
+
+
 int OTF_RStream_closeDefBuffer( OTF_RStream* rstream ) {
 
 
@@ -237,11 +266,9 @@ int OTF_RStream_closeDefBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->defBuffer );
 		if( 0 == ret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing defbuffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 
 		}
 		
@@ -264,20 +291,25 @@ OTF_RBuffer* OTF_RStream_getEventBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_EVENT, 0, NULL );
 		if( NULL == filename ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-					"OTF_getFilename() failed.\n",
-					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"OTF_getFilename() failed for event file with name stub '%s' and ID %u.\n",
+					__FUNCTION__, __FILE__, __LINE__, rstream->namestub, rstream->id );
 
 			return NULL;
 		}
 
 		rstream->eventBuffer= OTF_RBuffer_open( filename, rstream->manager );
 		free( filename );
+		filename = NULL;
 
 		if ( NULL == rstream->eventBuffer ) {
 
+/* *** commented because it can happen when file not exists
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"OTF_RBuffer_open() failed for filename '%s'.\n",
+					__FUNCTION__, __FILE__, __LINE__, filename );
+*/
+
 			return NULL;
 		}
 
@@ -301,11 +333,10 @@ int OTF_RStream_closeEventBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->eventBuffer );
 		if( 0 == ret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing event buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
+
 		}
 		rstream->eventBuffer= NULL;
 	}
@@ -326,17 +357,16 @@ OTF_RBuffer* OTF_RStream_getSnapsBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_SNAPS, 0, NULL );
 		if( NULL == filename ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 
 			return NULL;
 		}
 
 		rstream->snapsBuffer= OTF_RBuffer_open( filename, rstream->manager );
 		free( filename );
+		filename = NULL;
 
 		if ( NULL == rstream->snapsBuffer ) {
 
@@ -363,11 +393,10 @@ int OTF_RStream_closeSnapsBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->snapsBuffer );
 		if( 0 == ret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing snapshots buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
+
 		}
 		rstream->snapsBuffer= NULL;
 	}
@@ -388,11 +417,9 @@ OTF_RBuffer* OTF_RStream_getStatsBuffer( OTF_RStream* rstream ) {
 			rstream->id, OTF_FILETYPE_STATS, 0, NULL );
 		if( NULL == filename ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"OTF_getFilename() failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 
 			return NULL;
 		}
@@ -400,15 +427,10 @@ OTF_RBuffer* OTF_RStream_getStatsBuffer( OTF_RStream* rstream ) {
 
 		rstream->statsBuffer= OTF_RBuffer_open( filename, rstream->manager );
 		free( filename );
+		filename = NULL;
 
 		if ( NULL == rstream->statsBuffer ) {
 
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
-					"OTF_RBuffer_open() failed.\n",
-					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
-
 			return NULL;
 		}
 
@@ -432,11 +454,10 @@ int OTF_RStream_closeStatsBuffer( OTF_RStream* rstream ) {
 		ret&= OTF_RBuffer_close( rstream->statsBuffer );
 		if( 0 == ret ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"closing statistics buffer failed.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
+
 		}
 		rstream->statsBuffer= NULL;
 	}
@@ -445,34 +466,90 @@ int OTF_RStream_closeStatsBuffer( OTF_RStream* rstream ) {
 }
 
 
+OTF_RBuffer* OTF_RStream_getMarkerBuffer( OTF_RStream* rstream ) {
+
+
+	char* filename;
+
+
+	if ( NULL == rstream->markerBuffer ) {
+
+		filename= OTF_getFilename( rstream->namestub,
+			rstream->id, OTF_FILETYPE_MARKER, 0, NULL );
+		if( NULL == filename ) {
+			
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"OTF_getFilename() failed.\n",
+					__FUNCTION__, __FILE__, __LINE__ );
+
+			return NULL;
+		}
+
+		rstream->markerBuffer= OTF_RBuffer_open( filename, rstream->manager );
+		free( filename );
+		filename = NULL;
+
+		if ( NULL == rstream->markerBuffer ) {
+
+			return NULL;
+		}
+
+		OTF_RBuffer_setSize( rstream->markerBuffer, rstream->buffersizes );
+#ifdef HAVE_ZLIB
+		OTF_RBuffer_setZBufferSize( rstream->markerBuffer, rstream->zbuffersizes );
+#endif /* HAVE_ZLIB */
+	}
+
+	return rstream->markerBuffer;
+}
+
+
+int OTF_RStream_closeMarkerBuffer( OTF_RStream* rstream ) {
+
+
+	int ret= 1;
+	
+	if ( NULL != rstream->markerBuffer ) {
+
+		ret&= OTF_RBuffer_close( rstream->markerBuffer );
+		if( 0 == ret ) {
+			
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"closing defbuffer failed.\n",
+					__FUNCTION__, __FILE__, __LINE__ );
+
+		}
+		
+		rstream->markerBuffer= NULL;
+	}
+	
+	return ret;
+}
+
+
 void OTF_RStream_setBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 
 
 	if ( 50 > size ) {
 	
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"intended buffer size %u is too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
 		
 		return;
 
 	} else if ( 500 > size ) {
 	
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"buffer size %u is very small, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
 
 	} else if ( 10 * 1024 *1024 < size ) {
 
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"buffer size %u is rather big, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
+
 	}
 
 	rstream->buffersizes= size;
@@ -493,29 +570,24 @@ void OTF_RStream_setZBufferSizes( OTF_RStream* rstream, uint32_t size ) {
 	
 	if ( 32 > size ) {
 	
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"intended zbuffer size %u is too small, rejected.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
 		
 		return;
 
 	} else if ( 512 > size ) {
 	
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is very small, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
 
 	} else if ( 10 * 1024 *1024 < size ) {
 
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Warning( "WARNING in function %s, file: %s, line: %i:\n "
 				"zbuffer size %u is rather big, accepted though.\n",
 				__FUNCTION__, __FILE__, __LINE__, size );
-#		endif /* OTF_VERBOSE */
+
 	}
 
 	rstream->zbuffersizes= size;
@@ -540,12 +612,10 @@ void OTF_RStream_setRecordLimit( OTF_RStream* rstream, uint64_t limit ) {
 
 	if( limit == OTF_READ_ERROR ) {
 
-#		ifdef OTF_VERBOSE
-			fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+		OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 				"limit cannot be set to %llu. Reset to OTF_READ_MAXRECORDS.\n",
 				__FUNCTION__, __FILE__, __LINE__,
 				(long long unsigned) limit );
-#		endif /* OTF_VERBOSE */
 	
 		limit= OTF_READ_MAXRECORDS;
 	}
@@ -569,6 +639,7 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 
 	int ret;
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->defBuffer ) {
@@ -580,11 +651,9 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 
 		if ( NULL == rstream->defBuffer ) {
 		
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no def buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 			
 			/* there is no def buffer available for this stream */
 			return OTF_READ_ERROR;
@@ -600,14 +669,24 @@ uint64_t OTF_RStream_readDefinitions( OTF_RStream* rstream, OTF_HandlerArray* ha
 			return recordcount;
 		}
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->defBuffer->buffer + rstream->defBuffer->pos);
+
 		ret= OTF_Reader_parseDefRecord( rstream->defBuffer, handlers, rstream->id );
 		if ( 0 == ret ) {
 
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
-
-		recordcount++;
+	
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->defBuffer->list);
+			recordcount++;
+		}
+		
 	}
 
 	return recordcount;
@@ -626,6 +705,7 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->eventBuffer ) {
@@ -636,11 +716,9 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 		rstream->eventBuffer= OTF_RStream_getEventBuffer( rstream );
 		if( NULL == rstream->eventBuffer ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no event buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 			
 			return OTF_READ_ERROR;
 		}
@@ -667,7 +745,7 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			
 			if ( oldtime > currenttime ) {
 		
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -677,6 +755,10 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->eventBuffer->buffer + rstream->eventBuffer->pos);
+
 		ret= OTF_Reader_parseEventRecord( rstream->eventBuffer, handlers );
 		if ( 0 == ret ) {
 
@@ -684,7 +766,13 @@ uint64_t OTF_RStream_readEvents( OTF_RStream* rstream, OTF_HandlerArray* handler
 			return OTF_READ_ERROR;
 		}
 
-		recordcount++;
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->eventBuffer->list);
+			recordcount++;
+		}
+		
 	}
 
 	return recordcount;
@@ -703,6 +791,7 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->snapsBuffer ) {
@@ -713,11 +802,9 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 		rstream->snapsBuffer= OTF_RStream_getSnapsBuffer( rstream );
 		if( NULL == rstream->snapsBuffer ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no snapshots buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 			
 			return OTF_READ_ERROR;
 		}
@@ -744,7 +831,7 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			
 			if ( oldtime > currenttime ) {
 		
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -754,6 +841,9 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->snapsBuffer->buffer + rstream->snapsBuffer->pos);
 
 		ret= OTF_Reader_parseSnapshotsRecord( rstream->snapsBuffer, handlers );
 		if ( 0 == ret ) {
@@ -761,11 +851,16 @@ uint64_t OTF_RStream_readSnapshots( OTF_RStream* rstream, OTF_HandlerArray* hand
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
+		
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->snapsBuffer->list);
+			recordcount++;
+		}
 
-		recordcount++;
 	}
 
-
 	return recordcount;
 }
 
@@ -782,6 +877,7 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 		uint64_t oldtime= 0;
 #	endif
 
+	char next_char = '\0';
 
 	/* initialized? */
 	if ( NULL == rstream->statsBuffer ) {
@@ -792,11 +888,9 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 		rstream->statsBuffer= OTF_RStream_getStatsBuffer( rstream );
 		if( NULL == rstream->statsBuffer ) {
 			
-#			ifdef OTF_VERBOSE
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"the stream has no statistics buffer.\n",
 					__FUNCTION__, __FILE__, __LINE__ );
-#			endif /* OTF_VERBOSE */
 			
 			return OTF_READ_ERROR;
 		}
@@ -823,7 +917,7 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 
 			if ( oldtime > currenttime ) {
 		
-				fprintf( stderr, "ERROR in function %s, file: %s, line: %i:\n "
+				OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
 					"Time does decrease!!! %llu < %llu.\n",
 					__FUNCTION__, __FILE__, __LINE__,
 					(unsigned long long) currenttime,
@@ -833,6 +927,9 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 			}
 #		endif
 
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->statsBuffer->buffer + rstream->statsBuffer->pos);
 
 		ret= OTF_Reader_parseStatisticsRecord( rstream->statsBuffer, handlers );
 		if ( 0 == ret ) {
@@ -841,10 +938,77 @@ uint64_t OTF_RStream_readStatistics( OTF_RStream* rstream, OTF_HandlerArray* han
 			/* maybe later an errorhandler gives the record to the user */
 			return OTF_READ_ERROR;
 		}
+		
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->statsBuffer->list);
+			recordcount++;
+		}
+
 
-		recordcount++;
 	}
 
+	return recordcount;
+}
+
+
+uint64_t OTF_RStream_readMarker( OTF_RStream* rstream, OTF_HandlerArray* handlers ) {
+
+
+	uint64_t recordcount= 0;
+
+	int ret;
+
+	char next_char = '\0';
+
+	/* initialized? */
+	if ( NULL == rstream->markerBuffer ) {
+
+
+		/* init */
+
+		rstream->markerBuffer= OTF_RStream_getMarkerBuffer( rstream );
+
+		if ( NULL == rstream->markerBuffer ) {
+		
+			OTF_Error( "ERROR in function %s, file: %s, line: %i:\n "
+					"the stream has no marker buffer.\n",
+					__FUNCTION__, __FILE__, __LINE__ );
+			
+			/* there is no def buffer available for this stream */
+			return OTF_READ_ERROR;
+		}
+	}
+
+
+	while ( NULL != OTF_RBuffer_getRecord( rstream->markerBuffer ) ) {
+
+		if ( recordcount >= rstream->recordLimit ) {
+
+			/* record count limit reached, return */
+			return recordcount;
+		}
+
+		/* remember next record type, if it will be a none
+		   KEYVALUE record, dont't account it in recordcount */
+		next_char = *(rstream->markerBuffer->buffer + rstream->markerBuffer->pos);
+
+		ret= OTF_Reader_parseMarkerRecord( rstream->markerBuffer, handlers, rstream->id );
+		if ( 0 == ret ) {
+
+			/* maybe later an errorhandler gives the record to the user */
+			return OTF_READ_ERROR;
+		}
+
+		/* Now reset the KeyValue list, if we consumed a none
+		   KEYVALUE record */
+		if ( next_char != OTF_KEYWORD_F_KEYVALUE_PREFIX /* 'K' */ ) {
+			OTF_KeyValueList_reset(rstream->markerBuffer->list);
+			recordcount++;
+		}
+
+	}
 
 	return recordcount;
 }
