@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -17,10 +17,12 @@
 # define INITROU _init
 #endif
 
-extern void VT_Dyn_attach(void);
+#include "vt_dyninst.h"
+
 void INITROU(void);
 
 void INITROU()
 {
-  VT_Dyn_attach();
+  vt_dyn_attach();
 }
+
