@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -19,30 +19,41 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>
+
 #include "vt_comp.h"
+#include "vt_defs.h"
 #include "vt_env.h"
 #include "vt_error.h"
 #include "vt_inttypes.h"
 #include "vt_iowrap.h"
-#include "vt_memhook.h"
+#include "vt_mallocwrap.h"
 #include "vt_pform.h"
 #include "vt_trc.h"
 #include "vt_thrd.h"
 
 #if (defined(HAVE_DL) && HAVE_DL) && (defined(HAVE_DECL_RTLD_DEFAULT) && HAVE_DECL_RTLD_DEFAULT)
 # include <dlfcn.h>
-# define GET_ADDR_OF_UNDEF_FUNC(func) \
-  DEREF_IA64_FUNC_PTR(dlsym(RTLD_DEFAULT, (func)))
+# define GET_SO_FUNC_ADDR(func) \
+  GET_IA64_FUNC_ADDR(dlsym(RTLD_DEFAULT, (func)))
 #else /* HAVE_DL && HAVE_DECL_RTLD_DEFAULT */
-# define GET_ADDR_OF_UNDEF_FUNC(func) 0
+# define GET_SO_FUNC_ADDR(func) 0
 #endif /* HAVE_DL && HAVE_DECL_RTLD_DEFAULT */
 
 #ifdef __ia64__
-# define DEREF_IA64_FUNC_PTR(ptr) ((ptr) ? *(void**)(ptr) : (ptr))
+# define GET_IA64_FUNC_ADDR(addr) (long)((addr) ? *(void**)(addr) : (addr))
 #else /* __ia64__ */
-# define DEREF_IA64_FUNC_PTR(ptr) (ptr)
+# define GET_IA64_FUNC_ADDR(addr) (long)(addr)
 #endif /* __ia64__ */
 
+#define GET_THREAD_ID(tid) \
+  VT_CHECK_THREAD;         \
+  (tid) = VT_MY_THREAD
+
+#ifdef VT_COMPINST_CRAYCCE
+# define __cyg_profile_func_enter __pat_tp_func_entry
+# define __cyg_profile_func_exit  __pat_tp_func_return
+#endif /* VT_COMPINST_CRAYCCE */
+
 #define NM_LINE_BLK_LEN 1024
 #define NM_LINE_MAX_LEN 16384
 
@@ -118,7 +129,8 @@ static void get_symtab(void)
 
   uint8_t parse_error = 0;
 
-  VT_SUSPEND_IO_TRACING();
+  VT_SUSPEND_MALLOC_TRACING(VT_CURRENT_THREAD);
+  VT_SUSPEND_IO_TRACING(VT_CURRENT_THREAD);
 
   /* open nm-file, if given */
   nm_filename = vt_env_gnu_nmfile();
@@ -191,7 +203,7 @@ static void get_symtab(void)
     char  delim[2] = " ";
     int   nc = 1;
 
-    long  addr = -1;
+    long  addr = 0;
     char* filename = NULL;
     char* funcname = NULL;
     unsigned int lno = VT_NO_LNO;
@@ -239,6 +251,14 @@ static void get_symtab(void)
     if ( line[strlen(line)-1] == '\n' )
       line[strlen(line)-1] = '\0';
 
+    /* ignore line if it is empty */
+    if ( *line == '\0' )
+      continue;
+
+    /* ignore nm input file name */
+    if ( line[strlen(line)-1] == ':' )
+      continue;
+
     /* split line to columns */
     col = strtok(line, delim);
     do
@@ -246,12 +266,11 @@ static void get_symtab(void)
       if ( nc == 1 ) /* column 1 (address) */
       {
         /* if there is no address in the first column the symbol could be
-           undefined; try get its address later (nc==3) */
+           defined within a shared object; try get its address later (nc==3) */
         if ( strlen(col) == 1 )
         {
-          nc = 3;
-          strcpy(delim, "\t");
-          continue;
+          nc++; /* <- will be 3 in the next round */
+          *delim = '\t';
         }
         /* otherwise, convert address string */
         else
@@ -267,22 +286,27 @@ static void get_symtab(void)
           parse_error = 1;
           break;
         }
-        strcpy(delim, "\t");
+
+        *delim = '\t';
       }
       else if ( nc == 3 ) /* column 3 (symbol) */
       {
-        funcname = col;
-        strcpy(delim, ":");
+        long soaddr;
 
-        /* try to get address of undefined function, if necessary */
-        if ( addr == -1 )
-          addr = (long)GET_ADDR_OF_UNDEF_FUNC(funcname);
+        funcname = col;
+
+        /* the symbol might be defined within a shared object; try to get
+           its real address */
+        if ( ( soaddr = GET_SO_FUNC_ADDR(funcname) ) != 0 )
+          addr = soaddr;
 
         /* ignore function, if its address could not be determined */
         if ( addr == 0 )
           break;
+
+        *delim = ':';
       }
-      else if( nc == 4 ) /* column 4 (filename) */
+      else if ( nc == 4 ) /* column 4 (filename) */
       {
         filename = col;
       }
@@ -302,14 +326,14 @@ static void get_symtab(void)
     {
       break;
     }
-    /* at least three columns must be read (nc>=3) */
+    /* at least two columns must be read */
     else if ( nc < 3 )
     {
       parse_error = 1;
       break;
     }
-    /* add symbol to hash table, if we have its address */
-    else if ( addr > 0 )
+    /* add symbol to hash table, if we have its address and name */
+    else if ( addr > 0 && funcname )
     {
       char* n = strdup(funcname);
       char* p;
@@ -367,26 +391,28 @@ static void get_symtab(void)
 
   free(line);
 
-  VT_RESUME_IO_TRACING();
+  VT_RESUME_IO_TRACING(VT_CURRENT_THREAD);
+  VT_RESUME_MALLOC_TRACING(VT_CURRENT_THREAD);
 }
 
 /*
  * Register new region
  */
 
-static void register_region(HashNode* hn) {
+static void register_region(uint32_t tid, HashNode* hn) {
   uint32_t fid = VT_NO_ID;
   uint32_t lno = VT_NO_LNO;
 
   /* -- register file if available -- */
   if (hn->fname != NULL)
   {
-    fid = vt_def_scl_file(hn->fname);
+    fid = vt_def_scl_file(tid, hn->fname);
     lno = hn->lno;
   }
 
   /* -- register region and store region identifier -- */
-  hn->vtid = vt_def_region(hn->name, fid, lno, VT_NO_LNO, NULL, VT_FUNCTION);
+  hn->vtid = vt_def_region(tid, hn->name, fid, lno, VT_NO_LNO,
+                           NULL, VT_FUNCTION);
 }
 
 void gnu_finalize(void);
@@ -400,26 +426,27 @@ void __cyg_profile_func_exit(void* func, void* callsite);
 void gnu_finalize()
 {
   int i, idx_min, idx_max;
-  uint32_t min, max, n;
+  uint32_t min, max;
   double avg;
   min = 0xffffffff;
   max = 0;
   idx_min = idx_max = 0;
   avg = 0.0;
 
-  n = n_htab_entries;
   for( i = 0; i < HASH_MAX; i++ )
   {
     uint32_t n_bucket_entries = 0;
-    while( htab[i] )
+
+    struct HN* p = htab[i];
+    while( p )
     {
-      struct HN* next = htab[i]->next;
-      free(htab[i]->name);
-      if( htab[i]->fname ) free(htab[i]->fname);
-      free(htab[i]);
-      htab[i] = next;
-      n_htab_entries--;
       n_bucket_entries++;
+
+      /* Set assigned region id back to VT_NO_ID instead of freeing the
+         hash-node, because after a fork the hash-node will be re-used for the
+         child process. This implies a small/non-increasing memory leak. */
+      p->vtid = VT_NO_ID;
+      p = p->next;
     }
     if( n_bucket_entries < min ) {
       min = n_bucket_entries;
@@ -429,21 +456,19 @@ void gnu_finalize()
       max = n_bucket_entries;
       idx_max = i;
     }
-    vt_cntl_msg( 3, "Hash bucket %i had %u entries (%.1f/1000)", i, n_bucket_entries, ((double)n_bucket_entries*1000)/n );
+    vt_cntl_msg(3, "Hash bucket %i had %u entries (%.1f/1000)",
+                i, n_bucket_entries, ((double)n_bucket_entries*1000)/n_htab_entries);
   }
-  avg = (double)n / HASH_MAX;
+  avg = (double)n_htab_entries / HASH_MAX;
   vt_cntl_msg( 3, "Hash statistics:\n"
                   "\tNumber of entries: %u\n"
                   "\tMin bucket size:   %u (%.1f/1000) at index %i\n"
                   "\tMax bucket size:   %u (%.1f/1000) at index %i\n"
                   "\tAvg bucket size:   %.1f",
-                  n,
-                  min, ((double)min*1000)/n, idx_min,
-                  max, ((double)max*1000)/n, idx_max,
+                  n_htab_entries,
+                  min, ((double)min*1000)/n_htab_entries, idx_min,
+                  max, ((double)max*1000)/n_htab_entries, idx_max,
                   avg );
-  vt_assert( n_htab_entries==0 );
-
-  gnu_init = 1;
 }
 
 /*
@@ -452,48 +477,50 @@ void gnu_finalize()
  */
 
 void __cyg_profile_func_enter(void* func, void* callsite) {
-  void* funcptr;
+  long addr;
+  uint32_t tid;
   uint64_t time;
   HashNode* hn;
 
-  funcptr = DEREF_IA64_FUNC_PTR(func);
+  addr = GET_IA64_FUNC_ADDR(func);
 
   /* -- if not yet initialized, initialize VampirTrace -- */
   if ( gnu_init ) {
-    VT_MEMHOOKS_OFF();
     gnu_init = 0;
     vt_open();
     vt_comp_finalize = gnu_finalize;
     get_symtab();
-    VT_MEMHOOKS_ON();
   }
 
   /* -- if VampirTrace already finalized, return -- */
   if ( !vt_is_alive ) return;
 
-  VT_MEMHOOKS_OFF();
+  /* -- get calling thread id -- */
+  GET_THREAD_ID(tid);
+
+  VT_SUSPEND_MALLOC_TRACING(tid);
 
   time = vt_pform_wtime();
 
   /* -- get region identifier -- */
-  if ( (hn = hash_get((long)funcptr))) {
+  if ( (hn = hash_get(addr)) ) {
     if ( hn->vtid == VT_NO_ID ) {
       /* -- region entered the first time, register region -- */
 #if (defined(VT_MT) || defined(VT_HYB))
       VTTHRD_LOCK_IDS();
       if( hn->vtid == VT_NO_ID )
-        register_region(hn);
+        register_region(tid, hn);
       VTTHRD_UNLOCK_IDS();
 #else /* VT_MT || VT_HYB */
-      register_region(hn);
+      register_region(tid, hn);
 #endif /* VT_MT || VT_HYB */
     }
 
     /* -- write enter record -- */
-    vt_enter(&time, hn->vtid);
+    vt_enter(tid, &time, hn->vtid);
   }
 
-  VT_MEMHOOKS_ON();
+  VT_RESUME_MALLOC_TRACING(tid);
 }
 
 /*
@@ -502,22 +529,26 @@ void __cyg_profile_func_enter(void* func, void* callsite) {
  */
 
 void __cyg_profile_func_exit(void* func, void* callsite) {
-  void* funcptr;
+  long addr;
+  uint32_t tid;
   uint64_t time;
 
-  funcptr = DEREF_IA64_FUNC_PTR(func);
+  addr = GET_IA64_FUNC_ADDR(func);
 
   /* -- if VampirTrace already finalized, return -- */
   if ( !vt_is_alive ) return;
 
-  VT_MEMHOOKS_OFF();
+  /* -- get calling thread id -- */
+  GET_THREAD_ID(tid);
+
+  VT_SUSPEND_MALLOC_TRACING(tid);
 
   time = vt_pform_wtime();
 
   /* -- write exit record -- */
-  if ( hash_get((long)funcptr) ) {
-    vt_exit(&time);
+  if ( hash_get(addr) ) {
+    vt_exit(tid, &time);
   }
 
-  VT_MEMHOOKS_ON();
+  VT_RESUME_MALLOC_TRACING(tid);
 }
