@@ -50,18 +50,18 @@ EXTERN int    vt_env_stat_collop_dtls(void);
 EXTERN int    vt_env_snapshots(void);
 EXTERN int    vt_env_max_snapshots(void);
 EXTERN int    vt_env_verbose(void);
-EXTERN int    vt_env_debug(void);
 EXTERN int    vt_env_do_unify(void);
 EXTERN int    vt_env_do_clean(void);
-EXTERN int    vt_env_memtrace(void);
-EXTERN int    vt_env_memtrace_marker(void);
 EXTERN int    vt_env_cpuidtrace(void);
 EXTERN int    vt_env_iotrace(void);
 EXTERN int    vt_env_iotrace_extended(void);
 EXTERN char*  vt_env_iolibpathname(void);
-EXTERN int    vt_env_libctrace(void);
+EXTERN int    vt_env_exectrace(void);
+EXTERN int    vt_env_memtrace(void);
+EXTERN int    vt_env_memtrace_marker(void);
 EXTERN int    vt_env_omptrace(void);
 EXTERN int    vt_env_mpitrace(void);
+EXTERN int    vt_env_mpi_ignore_filter(void);
 EXTERN int    vt_env_mpicheck(void);
 EXTERN int    vt_env_mpicheck_errexit(void);
 EXTERN char*  vt_env_rusage(void);
