@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -21,10 +21,14 @@
 #include "vt_error.h"
 #include "vt_env.h"
 #include "vt_iowrap.h"
+#include "vt_trc.h"
 
 #define VT_MSG_PFIX  "VampirTrace"
 #define VT_MSG_SIZE  1024
 
+/* exit without performing exit handler (vt_close) on fatal error */
+#define DO_EXIT_FAILURE vt_failure = 1; exit(EXIT_FAILURE)
+
 static int vt_pid = -1;
 
 void vt_error_pid(const int pid)
@@ -50,12 +54,12 @@ static void vt_print_msg(const char* prefix, const char* fmt, va_list az)
   fflush(NULL);
 }
 
-void vt_assert_impl(const char* f, int l, const char* expr)
+void vt_libassert_fail(const char* f, int l, const char* expr)
 {
   vt_iowrap_externals_init();
   libc_fprintf(stderr, "%s: FATAL: %s:%d: Assertion `%s' failed\n\nPlease report this incident to " PACKAGE_BUGREPORT "\n",
 	       VT_MSG_PFIX, f, l, expr);
-  exit(EXIT_FAILURE);
+  DO_EXIT_FAILURE;
 }
 
 void vt_error_impl(const char* f, int l)
@@ -65,7 +69,8 @@ void vt_error_impl(const char* f, int l)
 
   perror(buffer);
   fflush(stderr);
-  exit(EXIT_FAILURE);
+  
+  DO_EXIT_FAILURE;
 }
 
 void vt_error_msg(const char* fmt, ...)
@@ -75,7 +80,8 @@ void vt_error_msg(const char* fmt, ...)
   va_start(ap, fmt);
   vt_print_msg("FATAL", fmt, ap);
   va_end(ap);
-  exit(EXIT_FAILURE);
+  
+  DO_EXIT_FAILURE;
 }
 
 void vt_warning(const char* fmt, ...)
@@ -100,14 +106,3 @@ void vt_cntl_msg(int level, const char* fmt, ...)
   }
 }
 
-void vt_debug_msg(int level, const char* fmt, ...)
-{
-  va_list ap;
-
-  if (vt_env_debug() >= level) {
-    va_start(ap, fmt);
-    vt_print_msg("DEBUG", fmt, ap);
-    va_end(ap);
-  }
-}
-
