@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -19,21 +19,25 @@
 #include <unistd.h>
 #include <errno.h>
 
-#include "otf.h"
 #include "papi.h"
 
+#include "vt_defs.h"
 #include "vt_env.h"
 #include "vt_error.h"
 #include "vt_inttypes.h"
+#include "vt_iowrap.h"
 #include "vt_metric.h"
 
-#if !(defined(HAVE_DECL_LONG_LONG) && HAVE_DECL_LONG_LONG)
+#if !(defined(HAVE_LONG_LONG) && HAVE_LONG_LONG)
 # define long_long long long
-#endif /* HAVE_DECL_LONG_LONG */
+#endif /* HAVE_LONG_LONG */
 
 #if PAPI_VER_CURRENT >= PAPI_VERSION_NUMBER(3,9,0,0)
 # define PAPIC
-#endif
+#endif /* PAPI_VER_CURRENT >= 3.9.0.0 */
+#if PAPI_VER_CURRENT >= PAPI_VERSION_NUMBER(5,0,0,0)
+# define PAPIV
+#endif /* PAPI_VER_CURRENT >= 5.0.0.0 */
 
 #ifndef TIMER_PAPI_REAL_CYC
 # define TIMER_PAPI_REAL_CYC 10
@@ -87,8 +91,8 @@ static struct metric* metricv[VT_METRIC_MAXNUM];
 static int    nmetrics = 0;
 
 static metricmap_t* metricmap_append(metricmap_t* map,
-				     metmap_t type,
-				     char* event, char* alias)
+                                     metmap_t type,
+                                     char* event, char* alias)
 {
   /*printf("Def 0x%X %s = <%s>\n", type, event, alias);*/
 
@@ -114,7 +118,7 @@ static metricmap_t* metricmap_append(metricmap_t* map,
   map->event_name = strdup(event);
   map->alias_name = strdup(alias);
   map->next = NULL;
-    
+
   return map;
 }
 
@@ -169,10 +173,10 @@ static metricmap_t* vt_metricmap_init(metmap_t match)
 
   if (!specfile) return NULL;
 
-  fp = fopen(specfile, "r"); 
+  fp = fopen(specfile, "r");
   if (fp == NULL) {
     vt_cntl_msg(2, "Failed to open metric specification %s: %s",
-		specfile, strerror(errno));
+                specfile, strerror(errno));
     return NULL;
   }
 
@@ -185,7 +189,7 @@ static metricmap_t* vt_metricmap_init(metmap_t match)
     while (len && ((line[len-1] == ' ') || (line[len-1] == '\t'))) len--;
     line[len] = '\0'; /* chop comment and return */
     lineno++;
-    if (len <= 1) continue; 
+    if (len <= 1) continue;
     defs++;
     if      (!strncmp("measure", line, 7)) type=METMAP_MEASURE;
     else if (!strncmp("compose", line, 7)) type=METMAP_COMPOSE;
@@ -207,16 +211,16 @@ static metricmap_t* vt_metricmap_init(metmap_t match)
     /*printf("Def %2d:<%s> %s <%s>\n", defs, def_name, line, def_args);*/
     len = strlen(def_args); /* length of definition arguments */
     if (((type == METMAP_MEASURE) && (match & METMAP_MEASURE)) ||
-	((type == METMAP_AGGROUP) && (match & METMAP_AGGROUP))) {
+        ((type == METMAP_AGGROUP) && (match & METMAP_AGGROUP))) {
       if (((int)strcspn(def_args, "=+") != len) ||
-	  (((int)strcspn(def_args, "=+-*/ \t") != len)
-	   && (type == METMAP_MEASURE))) {
-	type = METMAP_INVALID;
-	invalid_defs++;
-	vt_cntl_msg(2, "XXXX Def %d:%s <%s> invalid!", lineno, line, def_name);
+          (((int)strcspn(def_args, "=+-*/ \t") != len)
+           && (type == METMAP_MEASURE))) {
+        type = METMAP_INVALID;
+        invalid_defs++;
+        vt_cntl_msg(2, "XXXX Def %d:%s <%s> invalid!", lineno, line, def_name);
       } else {
-	map = metricmap_append(map, type, def_name, def_args);
-	measure_defs++;
+        map = metricmap_append(map, type, def_name, def_args);
+        measure_defs++;
       }
     } else if ((type == METMAP_COMPOSE) && (match & METMAP_COMPOSE)) {
        map = metricmap_append(map, type, def_name, def_args);
@@ -228,26 +232,26 @@ static metricmap_t* vt_metricmap_init(metmap_t match)
     if (mapv == NULL) mapv = map; /* initialise head of vector */
   }
   vt_cntl_msg(2, "Mapped %d/%d defs from \"%s\"",
-	      measure_defs+aggroup_defs+compose_defs+compute_defs, defs, specfile);
+              measure_defs+aggroup_defs+compose_defs+compute_defs, defs, specfile);
 #if 0
-  printf("measure %d aggroup %d compose %d compute %d unknown %d invalid %d\n", 
-	 measure_defs, aggroup_defs, compose_defs, compute_defs,
-	 unknown_defs, invalid_defs);
+  printf("measure %d aggroup %d compose %d compute %d unknown %d invalid %d\n",
+         measure_defs, aggroup_defs, compose_defs, compute_defs,
+         unknown_defs, invalid_defs);
 #endif
   fclose(fp);
   return mapv;
 }
 
-static void metricv_add(char* name, int code)
+static void metricv_add(char* name, int code, uint32_t props)
 {
   if (nmetrics >= VT_METRIC_MAXNUM) {
     vt_error_msg("Number of counters exceeds VampirTrace allowed maximum "
-		 "of %d", VT_METRIC_MAXNUM);
+                 "of %d", VT_METRIC_MAXNUM);
   } else {
     metricv[nmetrics] = (struct metric*)malloc(sizeof(struct metric));
     metricv[nmetrics]->name = strdup(name);
     metricv[nmetrics]->descr[0] = '\0';
-    metricv[nmetrics]->props = OTF_COUNTER_TYPE_ACC;
+    metricv[nmetrics]->props = props;
     metricv[nmetrics]->papi_code = code;
     nmetrics++;
   }
@@ -259,7 +263,11 @@ static void metric_error(int errcode, char *note)
 {
   char errstring[PAPI_MAX_STR_LEN];
 
+#ifdef PAPIV
+  PAPI_perror(errstring);
+#else
   PAPI_perror(errcode, errstring, PAPI_MAX_STR_LEN);
+#endif
   if (errcode == PAPI_ESYS) {
     strncat(errstring, ": ", PAPI_MAX_STR_LEN-strlen(errstring));
     strncat(errstring, strerror(errno), PAPI_MAX_STR_LEN-strlen(errstring));
@@ -273,7 +281,11 @@ static void metric_warning(int errcode, char *note)
 {
   char errstring[PAPI_MAX_STR_LEN];
 
+#ifdef PAPIV
+  PAPI_perror(errstring);
+#else
   PAPI_perror(errcode, errstring, PAPI_MAX_STR_LEN);
+#endif
   if (errcode == PAPI_ESYS) {
     strncat(errstring, ": ", PAPI_MAX_STR_LEN-strlen(errstring));
     strncat(errstring, strerror(errno), PAPI_MAX_STR_LEN-strlen(errstring));
@@ -295,31 +307,39 @@ static void metric_descriptions(void)
 
     if (strcmp(info.long_descr, metricv[i]->name) != 0) {
       strncpy(metricv[i]->descr, info.long_descr, sizeof(metricv[i]->descr));
-              
+
       /* tidy description if necessary */
       j=strlen(metricv[i]->descr)-1;
       if (metricv[i]->descr[j] == '\n') metricv[i]->descr[j]='\0';
       j=strlen(metricv[i]->descr)-1;
       if (metricv[i]->descr[j] != '.')
-	strncat(metricv[i]->descr, ".", sizeof(metricv[i]->descr));
+        strncat(metricv[i]->descr, ".",
+                sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
     }
 
     if (metricv[i]->papi_code & PAPI_PRESET_MASK) { /* PAPI preset */
       char *postfix_chp = info.postfix;
       char derive_ch = strcmp(info.derived,"DERIVED_SUB")?'+':'-';
-      strncat(metricv[i]->descr, " [ ", sizeof(metricv[i]->descr));
-      strncat(metricv[i]->descr, info.name[0], sizeof(metricv[i]->descr));
+      strncat(metricv[i]->descr, " [ ",
+              sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
+      strncat(metricv[i]->descr, info.name[0],
+              sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
       for (k=1; k<(int)info.count; k++) {
-	char op[4];
-	postfix_chp = postfix_chp?strpbrk(++postfix_chp, "+-*/"):NULL;
-	sprintf(op, " %c ", (postfix_chp?*postfix_chp:derive_ch));
-	strncat(metricv[i]->descr, op, sizeof(metricv[i]->descr));
-	strncat(metricv[i]->descr, info.name[k], sizeof(metricv[i]->descr));
+        char op[4];
+        postfix_chp = postfix_chp?strpbrk(++postfix_chp, "+-*/"):NULL;
+        sprintf(op, " %c ", (postfix_chp?*postfix_chp:derive_ch));
+        strncat(metricv[i]->descr, op,
+                sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
+        strncat(metricv[i]->descr, info.name[k],
+                sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
       }
-      strncat(metricv[i]->descr, " ]", sizeof(metricv[i]->descr));
+      strncat(metricv[i]->descr, " ]",
+              sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
       if (strcmp(info.symbol, metricv[i]->name) != 0) { /* add preset name */
-	strncat(metricv[i]->descr, " = ", sizeof(metricv[i]->descr)); 
-	strncat(metricv[i]->descr, info.symbol, sizeof(metricv[i]->descr));
+        strncat(metricv[i]->descr, " = ",
+                sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
+        strncat(metricv[i]->descr, info.symbol,
+                sizeof(metricv[i]->descr)-strlen(metricv[i]->descr));
       }
     }
 
@@ -332,7 +352,7 @@ static void metric_test(void)
 {
   int i, j;
   int retval;
-  
+
   int component;
   struct eventmap_t * EventSet[VT_METRIC_MAXNUM];
   for (i=0; i<VT_METRIC_MAXNUM; i++)
@@ -373,7 +393,7 @@ static void metric_test(void)
     retval = PAPI_cleanup_eventset(EventSet[i]->EventId);
     if ( retval != PAPI_OK )
       metric_error(retval, "PAPI_cleanup_eventset");
-  
+
     retval = PAPI_destroy_eventset(&(EventSet[i]->EventId));
     if ( retval != PAPI_OK )
       metric_error(retval, "PAPI_destroy_eventset");
@@ -390,12 +410,12 @@ int vt_metric_open()
   char* env_sep;
   char* var;
   char* token;
-  int forceprop;
+  char* saveptr;
   PAPI_event_info_t info;
   metricmap_t* mapv = NULL;
   metricmap_t* map;
 
-  /* read environment variable "VT_METRICS". Return if 
+  /* read environment variable "VT_METRICS". Return if
      uset and no PAPI timer used. */
   env = vt_env_metrics();
   if( env == NULL )
@@ -422,56 +442,60 @@ int vt_metric_open()
 
   var = strdup(env);
   vt_cntl_msg(2, "VT_METRICS=%s", var);
-        
+
   /* read metrics from specification string */
-  token = strtok(var, env_sep);
+  token = strtok_r(var, env_sep, &saveptr);
   while ( token && (nmetrics < VT_METRIC_MAXNUM) ) {
+    /* set counter properties */
+    uint32_t props;
     if (token[0]=='!')
     {
-      forceprop=1;
+      props = VT_CNTR_ABS | VT_CNTR_NEXT;
       token++;
     }
     else
-      forceprop=0;
+    {
+      props = VT_CNTR_ACC;
+    }
     /* search metricmap for a suitable definition */
     map = mapv;
     /*printf("Token%d: <%s>\n", nmetrics, token);*/
     while (map != NULL) {
       if ( strcmp(map->event_name, token) == 0 ) {
-	/*printf("Definition %s = <%s>\n", token, map->alias_name);*/
-	/* expand definition and set components */
-	char* c_token = map->alias_name;
-	int len = strcspn(c_token, " \t"); /* first token */
-	int got_valid_match = 1; /* to be verified */
-	int k = 0;
-	do { /* verify each component of definition is available */
-	  char component[64];
-	  int code = -1;
-	  strncpy(component, c_token, len);
-	  component[len] = '\0';
-	  /*printf("Comp[%d] <%s>\n", k, component);*/
-	  c_token += len + strspn(c_token+len, " \t");
-	  len = strcspn(c_token, " \t"); /* next token */
+        /*printf("Definition %s = <%s>\n", token, map->alias_name);*/
+        /* expand definition and set components */
+        char* c_token = map->alias_name;
+        int len = strcspn(c_token, " \t"); /* first token */
+        int got_valid_match = 1; /* to be verified */
+        int k = 0;
+        do { /* verify each component of definition is available */
+          char component[64];
+          int code = -1;
+          strncpy(component, c_token, len);
+          component[len] = '\0';
+          /*printf("Comp[%d] <%s>\n", k, component);*/
+          c_token += len + strspn(c_token+len, " \t");
+          len = strcspn(c_token, " \t"); /* next token */
 
-	  PAPI_event_name_to_code(component, &code);
-	  memset(&info, 0, sizeof(PAPI_event_info_t));
-	  retval = PAPI_get_event_info(code, &info);
-	  /*printf("v[%d] %s [0x%X] %d\n", k, component, code, info.count);*/
-       
-	  if (info.count == 0) {
-	    /*printf("Event %s *N/A*\n", component);*/
-	    got_valid_match = 0;
-	  } else if ((k==0) && (len==0)) { /* use provided event name */
-	    metricv_add(token, code);
-	  } else { /* use alias component name */
-	    metricv_add(component, code);
-	  }
-	  k++;
-	} while (got_valid_match && (len > 0));
-	if (got_valid_match) {
-	  /*printf("Definition %s = <%s> OK\n", map->event_name, map->alias_name);*/
-	  break; /* accept this event definition */
-	}
+          PAPI_event_name_to_code(component, &code);
+          memset(&info, 0, sizeof(PAPI_event_info_t));
+          retval = PAPI_get_event_info(code, &info);
+          /*printf("v[%d] %s [0x%X] %d\n", k, component, code, info.count);*/
+
+          if (info.count == 0) {
+            /*printf("Event %s *N/A*\n", component);*/
+            got_valid_match = 0;
+          } else if ((k==0) && (len==0)) { /* use provided event name */
+            metricv_add(token, code, props);
+          } else { /* use alias component name */
+            metricv_add(component, code, props);
+          }
+          k++;
+        } while (got_valid_match && (len > 0));
+        if (got_valid_match) {
+          /*printf("Definition %s = <%s> OK\n", map->event_name, map->alias_name);*/
+          break; /* accept this event definition */
+        }
       }
       map = map->next;
     }
@@ -482,20 +506,18 @@ int vt_metric_open()
       /*printf("Comp[X] <%s>\n", component);*/
       retval = PAPI_event_name_to_code(component, &code);
       if (retval != PAPI_OK || code == -1)
-	vt_error_msg("Metric <%s> not supported\n", component);
+        vt_error_msg("Metric <%s> not supported\n", component);
 
       memset(&info, 0, sizeof(PAPI_event_info_t));
       retval = PAPI_get_event_info(code, &info);
       /*printf("v[%d] %s [0x%X] %d\n", nmetrics, component, code, info.count);*/
       if (retval != PAPI_OK)
-	vt_error_msg("Metric <%s> not available\n", component);
+        vt_error_msg("Metric <%s> not available\n", component);
 
-      metricv_add(component, code);
+      metricv_add(component, code, props);
     }
 
-    if (forceprop)
-      metricv[nmetrics-1]->props=OTF_COUNTER_TYPE_ABS;
-    token = strtok(NULL, env_sep);
+    token = strtok_r(NULL, env_sep, &saveptr);
   }
 
   /*printf("nmetrics=%d\n", nmetrics);*/
@@ -517,7 +539,7 @@ int vt_metric_open()
 void vt_metric_close()
 {
   int i;
-  
+
   for ( i = 0; i < nmetrics; i++ ) {
     free (metricv[i]->name);
     free(metricv[i]);
@@ -589,7 +611,7 @@ struct vt_metv* vt_metric_create()
   return metv;
 }
 
-void vt_metric_free(struct vt_metv* metv)
+void vt_metric_free(struct vt_metv* metv, uint32_t tid)
 {
   int retval, i;
   long_long papi_vals[VT_METRIC_MAXNUM];
@@ -599,6 +621,8 @@ void vt_metric_free(struct vt_metv* metv)
 
   /* treat PAPI failures at this point as non-fatal */
 
+  VT_SUSPEND_IO_TRACING(tid);
+
   /* foreach used eventset */
   for (i=0; i < VT_METRIC_MAXNUM && metv->EventSet[i]!=NULL; i++)
   {
@@ -615,6 +639,9 @@ void vt_metric_free(struct vt_metv* metv)
     }
     free(metv->EventSet[i]);
   }
+
+  VT_RESUME_IO_TRACING(tid);
+
   free(metv);
 }
 
@@ -625,7 +652,7 @@ void vt_metric_thread_init(long (*id_fn)(void))
   if ( nmetrics == 0 )
     return;
 
-  retval = PAPI_thread_init((unsigned long (*)(void))(id_fn)); 
+  retval = PAPI_thread_init((unsigned long (*)(void))(id_fn));
   if ( retval != PAPI_OK)
     metric_error(retval, "PAPI_thread_init");
   vt_cntl_msg(2, "PAPI thread support initialized");
@@ -649,6 +676,8 @@ void vt_metric_read(struct vt_metv* metv, uint64_t offsets[],
   if ( metv == NULL )
     return;
 
+  VT_SUSPEND_IO_TRACING(VT_CURRENT_THREAD);
+
   /* foreach used eventset */
   for (i=0; i < VT_METRIC_MAXNUM && metv->EventSet[i]!=NULL; i++)
   {
@@ -663,6 +692,8 @@ void vt_metric_read(struct vt_metv* metv, uint64_t offsets[],
   else
     for ( i = 0; i < nmetrics; i++ )
       values[i] = (uint64_t) *metv->Values[i];
+
+  VT_RESUME_IO_TRACING(VT_CURRENT_THREAD);
 }
 
 int vt_metric_num()
@@ -700,12 +731,12 @@ uint64_t vt_metric_clckrt(void)
 
   if (!PAPI_is_initialized()) {
     /* initialize PAPI, since it hasn't already been initialized */
-    int retval = PAPI_library_init(PAPI_VER_CURRENT);  
+    int retval = PAPI_library_init(PAPI_VER_CURRENT);
     if ( retval != PAPI_VER_CURRENT )
       metric_error(retval, "PAPI_library_init");
   }
 
-  hwinfo = PAPI_get_hardware_info(); 
+  hwinfo = PAPI_get_hardware_info();
   if ( hwinfo == NULL)
     vt_error_msg("Failed to access PAPI hardware info\n");
   vt_cntl_msg(2, "Clock rate: %f MHz", hwinfo->mhz);
