@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -16,8 +16,6 @@
 #include "vt_error.h"
 #include "vt_pform.h"
 
-#include "util/hash.h"
-
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
@@ -55,8 +53,8 @@ static long vt_node_id = 0;
 /* platform specific initialization */
 void vt_pform_init() {
   int  pid = getpid();
-  char exec_proc[512];
-  char exec[1024];
+  char exec_proc[VT_PATH_MAX];
+  char exec[VT_PATH_MAX];
   int  exec_len;
   int  hostid_retries;
 
@@ -66,7 +64,7 @@ void vt_pform_init() {
 
   /* get full path of executable */
   snprintf(exec_proc, sizeof (exec_proc), VT_PROCDIR"%d/path/a.out", pid);
-  exec_len = readlink(exec_proc, exec, sizeof (exec));
+  exec_len = readlink(exec_proc, exec, sizeof (exec)-1);
   if(exec_len != -1)
   {
     exec[exec_len] = '\0';
@@ -79,16 +77,8 @@ void vt_pform_init() {
     vt_node_id = gethostid();
   }
   if (!vt_node_id)
-  {
     vt_error_msg("Maximum retries (%i) for gethostid exceeded!",
-		 VT_MAX_GETHOSTID_RETRIES);
-  }
-  else
-  {
-    char* nodename = vt_pform_node_name();
-    vt_node_id = (long)vt_hash((uint8_t*)nodename, strlen(nodename),
-			       (uint32_t)vt_node_id);
-  }
+                 VT_MAX_GETHOSTID_RETRIES);
 }
 
 /* directory of global file system  */
@@ -113,11 +103,11 @@ char* vt_pform_exec() {
 /* clock resolution */
 uint64_t vt_pform_clockres() {
 #if TIMER == TIMER_GETHRTIME
-  return 1e9;
+  return 1000000000LL;
 #elif TIMER == TIMER_PAPI_REAL_CYC
   return vt_metric_clckrt();
 #elif TIMER == TIMER_PAPI_REAL_USEC
-  return 1e6;
+  return 1000000LL;
 #endif
 }
 
