@@ -2,7 +2,7 @@
  * VampirTrace
  * http://www.tu-dresden.de/zih/vampirtrace
  *
- * Copyright (c) 2005-2010, ZIH, TU Dresden, Federal Republic of Germany
+ * Copyright (c) 2005-2012, ZIH, TU Dresden, Federal Republic of Germany
  *
  * Copyright (c) 1998-2005, Forschungszentrum Juelich, Juelich Supercomputing
  *                          Centre, Federal Republic of Germany
@@ -26,13 +26,22 @@
 #include "vt_trc.h"
 #include "vt_iowrap.h"
 
+
+#if defined(VT_PLUGIN_CNTR)
+# include "vt_plugin_cntr_int.h"
+#endif /* VT_PLUGIN_CNTR */
+
+
 /* vector of the thread objects */
 VTThrd** VTThrdv = NULL;
 
 /* number of thread objects */
 uint32_t VTThrdn = 0;
 
-/* mutexes for locking */
+/* maximum number of threads */
+uint32_t VTThrdMaxNum = 0;
+
+/* predefined mutexes for locking */
 #if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
 VTThrdMutex* VTThrdMutexEnv = NULL;
 VTThrdMutex* VTThrdMutexIds = NULL;
@@ -40,8 +49,11 @@ VTThrdMutex* VTThrdMutexIds = NULL;
 
 void VTThrd_init()
 {
+  /* get the maximum number of threads */
+  VTThrdMaxNum = (uint32_t)vt_env_max_threads();
+
   /* create vector of the thread objects */
-  VTThrdv = (VTThrd**)calloc(vt_env_max_threads(), sizeof(VTThrd*));
+  VTThrdv = (VTThrd**)calloc(VTThrdMaxNum, sizeof(VTThrd*));
   if ( VTThrdv == NULL )
     vt_error();
 
@@ -66,9 +78,9 @@ void VTThrd_init()
      (for Java this will be done in VTThrd_initJava(),
       'cause it gets the read thread name) */
 #if !defined(VT_JAVA)
-  VTThrdv[0] = VTThrd_create(0, 0, NULL);
-  VTThrd_open(VTThrdv[0], 0);
-#endif
+  VTThrd_create(NULL, 0, 0);
+  VTThrd_open(0);
+#endif /* VT_JAVA */
 }
 
 void VTThrd_finalize()
@@ -83,9 +95,13 @@ void VTThrd_finalize()
     free(VTThrdv);
 }
 
-VTThrd* VTThrd_create(uint32_t tid, uint32_t ptid, const char* tname)
+uint32_t VTThrd_create(const char* tname, uint32_t ptid, uint8_t is_virtual)
 {
-  VTThrd *thread;
+  VTThrd* thrd;
+
+  uint32_t tid;
+  uint32_t child_no = 0;
+
 #if defined(VT_METR)
   uint32_t num_metrics = (uint32_t)vt_metric_num();
 #endif /* VT_METR */
@@ -97,143 +113,220 @@ VTThrd* VTThrd_create(uint32_t tid, uint32_t ptid, const char* tname)
   VTTHRD_LOCK_ENV();
 #endif /* VT_MT || VT_HYB || VT_JAVA */
 
-  if (VTThrdn > (uint32_t)vt_env_max_threads())
+  /* create new thread ID */
+
+  tid = VTThrdn++;
+  if ( VTThrdn > VTThrdMaxNum )
   {
 #if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
     VTTHRD_UNLOCK_ENV();
 #endif /* VT_MT || VT_HYB || VT_JAVA */
-    vt_error_msg("Cannot create more than %d threads", vt_env_max_threads());
+    vt_error_msg("Cannot create more than %d threads", VTThrdMaxNum);
   }
+
+  /* get child number (thread name suffix) */
+  if ( tid != 0 )
+    child_no = ++(VTThrdv[ptid]->child_num);
+
 #if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
   VTTHRD_UNLOCK_ENV();
 #endif /* VT_MT || VT_HYB || VT_JAVA */
 
-  thread = (VTThrd*)calloc(1, sizeof(VTThrd));
-  if ( thread == NULL )
+  thrd = (VTThrd*)calloc(1, sizeof(VTThrd));
+  if ( thrd == NULL )
     vt_error();
 
-  /* set external thread name, if available */
-  if (tname)
-    strncpy(thread->name_extern, tname, sizeof(thread->name_extern)-1);
-
-  /* set thread's name prefix */
-  snprintf(thread->name_prefix, sizeof(thread->name_prefix)-1, "%s",
-           tid == 0 ? "Process" : "Thread");
-
-  /* set thread's name suffix */
-  if (tid != 0)
+  /* set thread name, if available */
+  if ( tname == NULL )
   {
-#if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
-    VTTHRD_LOCK_ENV();
-#endif /* VT_MT || VT_HYB || VT_JAVA */
-
-    snprintf(thread->name_suffix, sizeof(thread->name_suffix)-1, "%s:%d",
-             VTThrdv[ptid]->name_suffix, ++(VTThrdv[ptid]->child_num));
-
-#if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
-    VTTHRD_UNLOCK_ENV();
-#endif /* VT_MT || VT_HYB || VT_JAVA */
+    if ( tid == 0 ) tname = "Process";
+    else tname = "Thread";
   }
 
-  /* set parent ID of thread */
-  thread->parent_tid = ptid;
+  /* set thread name */
+  strncpy(thrd->name, tname, sizeof(thrd->name));
+  thrd->name[sizeof(thrd->name)-1] = '\0';
+
+  /* set thread name suffix */
+  if ( tid != 0 )
+  {
+    snprintf(thrd->name_suffix, sizeof(thrd->name_suffix)-1, "%s:%d",
+             VTThrdv[ptid]->name_suffix, child_no);
+  }
+
+  /* set thread ID */
+  thrd->tid = tid;
+
+  /* set parent thread ID */
+  thrd->parent_tid = ptid;
+
+  /* set the virtual thread flag */
+  thrd->is_virtual = is_virtual;
 
 #if defined(VT_GETCPU)
-  thread->cpuid_val = (uint32_t)-1;
+  thrd->cpuid_val = (uint32_t)-1;
 #endif /* VT_GETCPU */
 
 #if defined(VT_RUSAGE)
-  if (num_rusage > 0) {
+  if ( num_rusage > 0 && !is_virtual )
+  {
     /* create rusage object */
-    thread->ru_obj = vt_rusage_create();
+    thrd->ru_obj = vt_rusage_create();
 
     /* initialize per-thread arrays for rusage counter values */
-    thread->ru_valv = (uint64_t*)calloc(num_rusage, sizeof(uint64_t));
-    if ( thread->ru_valv == NULL )
+    thrd->ru_valv = (uint64_t*)calloc(num_rusage, sizeof(uint64_t));
+    if ( thrd->ru_valv == NULL )
       vt_error();
 
     /* initialize next timestamp for reading rusage counters */
-    thread->ru_next_read = 0;
+    thrd->ru_next_read = 0;
   }
 #endif /* VT_RUSAGE */
 
 #if defined(VT_METR)
-  if (num_metrics > 0) {
+  if ( num_metrics > 0 && !is_virtual )
+  {
     /* create event set */
-    thread->metv = vt_metric_create();
+    thrd->metv = vt_metric_create();
 
 # if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
     /* initialize per-thread arrays for counter offsets */
-    thread->offv = (uint64_t*)calloc(num_metrics, sizeof(uint64_t));
-    if ( thread->offv == NULL )
+    thrd->offv = (uint64_t*)calloc(num_metrics, sizeof(uint64_t));
+    if ( thrd->offv == NULL )
       vt_error();
 #endif /* VT_MT || VT_HYB || VT_JAVA */
 
     /* initialize per-thread arrays for counter values */
-    thread->valv = (uint64_t*)calloc(num_metrics, sizeof(uint64_t));
-    if ( thread->valv == NULL )
+    thrd->valv = (uint64_t*)calloc(num_metrics, sizeof(uint64_t));
+    if ( thrd->valv == NULL )
       vt_error();
   }
 #endif /* VT_METR */
 
 #if !defined(VT_DISABLE_RFG)
   /* initialize region filter and grouping management */
-  thread->rfg_regions = RFG_Regions_init();
+  if( tid == 0 )
+    thrd->rfg_regions = RFG_Regions_init();
+  else
+    thrd->rfg_regions = RFG_Regions_dup( VTThrdv[0]->rfg_regions );
 
-  if( thread->rfg_regions == NULL )
+  if( thrd->rfg_regions == NULL )
     vt_error_msg("Could not initialize region filter and grouping management");
+
+  /* initialize call stack level where recursive filtering was enabled */
+  thrd->stack_level_at_recfilt_enabled = -1;
 #endif /* VT_DISABLE_RFG */
 
   /* enable tracing */
-  thread->trace_status = VT_TRACE_ON;
+  thrd->trace_status = VT_TRACE_ON;
 
-  /* increment the thread object counter (for successful creations) */
+  VTThrdv[tid] = thrd;
 
 #if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
   VTTHRD_LOCK_ENV();
-  VTThrdn++;
+#endif /* VT_MT || VT_HYB || VT_JAVA */
   vt_cntl_msg(2, "Thread object #%u created, total number is %u",
-                 tid, VTThrdn);
+              tid, VTThrdn);
+#if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
   VTTHRD_UNLOCK_ENV();
-#else /* VT_MT || VT_HYB || VT_JAVA */
-  VTThrdn++;
-  vt_cntl_msg(2, "Thread object #%u created, total number is %u",
-                 tid, VTThrdn);
 #endif /* VT_MT || VT_HYB || VT_JAVA */
 
-  return thread;
+  return tid;
 }
 
-void VTThrd_open(VTThrd* thrd, uint32_t tid)
+void VTThrd_open(uint32_t tid)
 {
+  VTThrd* thrd = VTThrdv[tid];
   size_t bsize = vt_env_bsize();
 #if (defined(VT_MT) || defined(VT_HYB) || defined(VT_JAVA))
-  if (tid == 0) { /* master thread gets most buffer space */
-    bsize = (bsize / 10) * 7;
-  } else {        /* worker threads get less buffer space */
-    bsize = (bsize / 10);
+  size_t tbsize = vt_env_thread_bsize();
+  if( tbsize != 0 )
+  {
+    if( tid != 0 )
+      bsize = tbsize;
+  }
+  else
+  {
+    if( tid == 0 ) /* master thread gets most buffer space */
+      bsize = (bsize / 10) * 7;
+    else           /* worker threads get less buffer space */
+      bsize = (bsize / 10);
   }
 #endif /* VT_MT || VT_HYB || VT_JAVA */
 
-  if (thrd)
-  {
-    thrd->gen = VTGen_open(thrd->name_prefix, thrd->name_suffix,
-                           thrd->name_extern, thrd->parent_tid, tid, bsize);
-  }
+  /* create trace buffer */
+  thrd->gen = VTGen_open(thrd->name, thrd->name_suffix,
+                         thrd->parent_tid, tid, bsize);
 
+  /* MPI stuff, I/O, and plugin counter not available for
+     virtual threads (e.g. GPU); return */
+  if ( tid != 0 && thrd->is_virtual )
+    return;
+
+#if (defined (VT_MPI) || defined (VT_HYB))
+  /* initialize actual mode of MPI tracing operation */
+  thrd->mpi_tracing_enabled = vt_env_mpitrace();
+  /* initialize first matching ID for MPI collective ops. */
+  thrd->mpicoll_next_matchingid = 1;
+#endif /* VT_MPI || VT_HYB */
+
+#if (defined (VT_IOWRAP) || (defined(HAVE_MPI2_IO) && HAVE_MPI2_IO))
+  /* initialize first matching ID and handle */
+  thrd->io_next_matchingid = 1;
+  thrd->io_next_handle = 1;
+#endif /* VT_IOWRAP || HAVE_MPI2_IO */
 #if defined(VT_IOWRAP)
-  if (vt_env_iotrace())
+  if ( vt_env_iotrace() )
   {
     vt_iowrap_init();
     VT_ENABLE_IO_TRACING();
   }
 #endif /* VT_IOWRAP */
+
+#if defined(VT_EXECWRAP)
+  /* initialize actual mode of EXEC tracing operation */
+  thrd->exec_tracing_state = thrd->exec_tracing_enabled =
+    (uint8_t)vt_env_exectrace();
+#endif /* VT_EXECWRAP */
+
+#if defined(VT_MALLOCWRAP)
+  /* initialize actual mode of MALLOC tracing operation */
+  thrd->malloc_tracing_state = thrd->malloc_tracing_enabled =
+    (uint8_t)vt_env_memtrace();
+#endif /* VT_MALLOCWRAP */
+
+#if defined(VT_PLUGIN_CNTR)
+  /* if we really use plugins */
+  if ( vt_plugin_cntr_used && tid != 0 )
+  {
+    /* if this is no dummy thread */
+    if ( !vt_plugin_cntr_is_registered_monitor_thread() )
+    {
+      vt_plugin_cntr_thread_init(thrd, tid);
+
+      /* if this thread uses plugins */
+      if ( thrd->plugin_cntr_defines )
+        vt_plugin_cntr_thread_enable_counters(thrd);
+    }
+  }
+#endif /* VT_PLUGIN_CNTR */
+
+  /* if MPI-rank is disabled, switch tracing off for this thread */
+  if( vt_my_trace_is_disabled )
+    vt_trace_off(tid, 0, 1);
 }
 
 void VTThrd_close(VTThrd* thrd)
 {
-  if (thrd)
-    VTGen_close(thrd->gen);
+  if ( !thrd ) return;
+  
+#if defined(VT_PLUGIN_CNTR)
+  if ( vt_plugin_cntr_used && thrd->plugin_cntr_defines ){
+    /* then write the post mortem counters */
+    vt_plugin_cntr_write_post_mortem(thrd);
+  }
+#endif /* VT_PLUGIN_CNTR */
+  VTGen_close(thrd->gen);
 }
 
 void VTThrd_delete(VTThrd* thrd, uint32_t tid)
@@ -241,32 +334,22 @@ void VTThrd_delete(VTThrd* thrd, uint32_t tid)
   if ( !thrd ) return;
 
 #if !defined(VT_DISABLE_RFG)
-  /* write list of regions whose call limit are reached */
   if ( thrd->rfg_regions )
-  {
-    char* filter_filename;
-
-    filter_filename = (char*)calloc(VT_PATH_MAX + 1, sizeof(char));
-    if ( filter_filename == NULL )
-      vt_error();
-
-    snprintf(filter_filename, VT_PATH_MAX, "%s/%s.%x.filt",
-             vt_env_gdir(), vt_env_fprefix(),
-             65536*tid+(vt_my_trace+1));
-
-    RFG_Regions_dumpFilteredRegions(thrd->rfg_regions, filter_filename);
-
     RFG_Regions_free(thrd->rfg_regions);
-
-    free(filter_filename);
-  }
 #endif /* VT_DISABLE_RFG */
 
+  /* must be called before VTGen_delete */
+#if defined(VT_PLUGIN_CNTR)
+  /* if we really use plugins and this thread also uses some */
+  if ( vt_plugin_cntr_used && thrd->plugin_cntr_defines && !(thrd->is_virtual) )
+    vt_plugin_cntr_thread_exit(thrd);
+#endif /* VT_PLUGIN_CNTR */
+
   if ( thrd->gen )
     VTGen_delete(thrd->gen);
 
 #if defined(VT_RUSAGE)
-  if ( vt_rusage_num() > 0 )
+  if ( vt_rusage_num() > 0 && !(thrd->is_virtual) )
   {
     if ( thrd->ru_obj )
     {
@@ -282,11 +365,11 @@ void VTThrd_delete(VTThrd* thrd, uint32_t tid)
 #endif /* VT_RUSAGE */
 
 #if defined(VT_METR)
-  if ( vt_metric_num() > 0 )
+  if ( vt_metric_num() > 0 && !(thrd->is_virtual) )
   {
     if ( thrd->metv )
     {
-      vt_metric_free(thrd->metv);
+      vt_metric_free(thrd->metv, tid);
       thrd->metv = NULL;
     }
     if ( thrd->offv )
@@ -301,7 +384,6 @@ void VTThrd_delete(VTThrd* thrd, uint32_t tid)
     }
   }
 #endif /* VT_METR */
-
   free(thrd);
 
   /* decrement the thread object counter */
@@ -313,19 +395,19 @@ void VTThrd_delete(VTThrd* thrd, uint32_t tid)
 #else /* VT_MT || VT_HYB || VT_JAVA */
   VTThrdn--;
   vt_cntl_msg(2, "Thread object #%u deleted, leaving %u", tid, VTThrdn);
-#endif
+#endif /* VT_MT || VT_HYB || VT_JAVA */
 }
 
 void VTThrd_destroy(VTThrd* thrd, uint32_t tid)
 {
 #if !defined(VT_DISABLE_RFG)
   RFG_Regions_free(thrd->rfg_regions);
-#endif
+#endif /* VT_DISABLE_RFG */
 
   VTGen_destroy(thrd->gen);
 
 #if defined(VT_RUSAGE)
-  if ( vt_rusage_num() > 0 )
+  if ( vt_rusage_num() > 0 && !(thrd->is_virtual) )
   {
     if ( thrd->ru_obj )
     {
@@ -341,11 +423,11 @@ void VTThrd_destroy(VTThrd* thrd, uint32_t tid)
 #endif /* VT_RUSAGE */
 
 #if defined(VT_METR)
-  if ( vt_metric_num() > 0 )
+  if ( vt_metric_num() > 0 && !(thrd->is_virtual) )
   {
     if ( thrd->metv )
     {
-      vt_metric_free(thrd->metv);
+      vt_metric_free(thrd->metv, tid);
       thrd->metv = NULL;
     }
     if ( thrd->offv )
