@@ -9,7 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
- * Copyright (c) 2008      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2008-2009 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -26,7 +26,6 @@
 #include "ompi_config.h"
 
 #include "ompi/constants.h"
-#include "ompi/communicator/communicator.h"
 #include "ompi/mca/coll/coll.h"
 #include "orte/util/show_help.h"
 #include "coll_sm.h"
@@ -99,7 +98,7 @@ mca_coll_sm_component_t mca_coll_sm_component = {
 
     /* (default) number of segments for each communicator in the
        per-communicator shmem segment */
-    128,
+    8,
 
     /* (default) fragment size */
     8192,
@@ -181,13 +180,13 @@ static int sm_register(void)
                            cs->sm_tree_degree,
                            &cs->sm_tree_degree);
     if (cs->sm_tree_degree > cs->sm_control_size) {
-        orte_show_help("help-coll-sm.txt", 
+        orte_show_help("help-mpi-coll-sm.txt", 
                        "tree-degree-larger-than-control", true,
                        cs->sm_tree_degree, cs->sm_control_size);
         cs->sm_tree_degree = cs->sm_control_size;
     }
     if (cs->sm_tree_degree > 255) {
-        orte_show_help("help-coll-sm.txt", 
+        orte_show_help("help-mpi-coll-sm.txt", 
                        "tree-degree-larger-than-255", true,
                        cs->sm_tree_degree);
         cs->sm_tree_degree = 255;
