@@ -61,15 +61,24 @@ int ompi_mtl_mxm_irecv(struct mca_mtl_base_module_t* mtl,
 
     /* prepare a receive request embedded in the MTL request */
     mxm_recv_req = &mtl_mxm_request->mxm.recv;
+#if MXM_API >= MXM_VERSION(2,0)
+    mtl_mxm_request->is_send = 0;
+#endif
 
     mxm_recv_req->base.state               = MXM_REQ_NEW;
     ompi_mtl_mxm_set_recv_envelope(mxm_recv_req, comm, src, tag);
 
+#if MXM_API < MXM_VERSION(2,0)
     mxm_recv_req->base.flags               = 0;
+#endif
     mxm_recv_req->base.data_type           = MXM_REQ_DATA_BUFFER;
     mxm_recv_req->base.data.buffer.ptr     = mtl_mxm_request->buf;
     mxm_recv_req->base.data.buffer.length  = mtl_mxm_request->length;
+#if MXM_API < MXM_VERSION(1,5)
     mxm_recv_req->base.data.buffer.mkey    = MXM_MKEY_NONE;
+#else
+    mxm_recv_req->base.data.buffer.memh    = MXM_INVALID_MEM_HANDLE;
+#endif
     mxm_recv_req->base.context             = mtl_mxm_request;
     mxm_recv_req->base.completed_cb        = ompi_mtl_mxm_recv_completion_cb;
 
